@@ -394,6 +394,7 @@ private:
             if (!missingOutputPortWarningPrinted)
             {
                 std::cerr << "No valid MIDI output port was specified for some of the messages" << std::endl;
+                setApplicationReturnValue(EXIT_FAILURE);
                 missingOutputPortWarningPrinted = true;
             }
         }
@@ -406,29 +407,34 @@ private:
             case NONE:
                 break;
             case LIST:
-                for (auto&& device : MidiOutput::getDevices())
+                for (auto&& device : MidiOutput::getAvailableDevices())
                 {
-                    std::cout << device << std::endl;
+                    std::cout << device.name << std::endl;
                 }
                 break;
             case DEVICE:
             {
                 midiOut_ = nullptr;
                 midiOutName_ = cmd.opts_[0];
-                int index = MidiOutput::getDevices().indexOf(midiOutName_);
-                if (index >= 0)
+                auto devices = MidiOutput::getAvailableDevices();
+                for (int i = 0; i < devices.size(); ++i)
                 {
-                    midiOut_ = MidiOutput::openDevice(index);
+                    if (devices[i].name == midiOutName_)
+                    {
+                        midiOut_ = MidiOutput::openDevice(devices[i].identifier);
+                        midiOutName_ = devices[i].name;
+                        break;
+                    }
                 }
-                else
+                
+                if (midiOut_ == nullptr)
                 {
-                    StringArray devices = MidiOutput::getDevices();
                     for (int i = 0; i < devices.size(); ++i)
                     {
-                        if (devices[i].containsIgnoreCase(midiOutName_))
+                        if (devices[i].name.containsIgnoreCase(midiOutName_))
                         {
-                            midiOut_ = MidiOutput::openDevice(i);
-                            midiOutName_ = devices[i];
+                            midiOut_ = MidiOutput::openDevice(devices[i].identifier);
+                            midiOutName_ = devices[i].name;
                             break;
                         }
                     }
@@ -436,6 +442,7 @@ private:
                 if (midiOut_ == nullptr)
                 {
                     std::cerr << "Couldn't find MIDI output port \"" << midiOutName_ << "\"" << std::endl;
+                    setApplicationReturnValue(EXIT_FAILURE);
                 }
                 break;
             }
@@ -451,6 +458,7 @@ private:
                 if (midiOut_ == nullptr)
                 {
                     std::cerr << "Couldn't create virtual MIDI output port \"" << name << "\"" << std::endl;
+                    setApplicationReturnValue(EXIT_FAILURE);
                 }
                 else
                 {
@@ -458,6 +466,7 @@ private:
                 }
 #else
                 std::cerr << "Virtual MIDI output ports are not supported on Windows" << std::endl;
+                setApplicationReturnValue(EXIT_FAILURE);
 #endif
                 break;
             }
@@ -486,6 +495,7 @@ private:
                 else
                 {
                     std::cerr << "Couldn't find file \"" << path << "\"" << std::endl;
+                    setApplicationReturnValue(EXIT_FAILURE);
                 }
                 break;
             }
@@ -527,6 +537,7 @@ private:
                 if (number >= 32)
                 {
                     std::cerr << "Can't send 14bit MIDI CC for number " << number << " (it has to be smaller than 32)" << std::endl;
+                    setApplicationReturnValue(EXIT_FAILURE);
                 }
                 else
                 {
@@ -590,7 +601,7 @@ private:
                 float bpm = float(jlimit(1, 999, asDecOrHexIntValue(cmd.opts_[0])));
                 float msPerTick = (60.f * 1000.f / bpm) / 24.f;
                 sendMidiMessage(MidiMessage::midiClock());
-                for (int ticks = 1; ticks <= 24 * 2; ++ticks)
+                for (int ticks = 1; ticks < 24 * 2; ++ticks)
                 {
                     Time::waitForMillisecondCounter(now + uint32(float(ticks) * msPerTick));
                     sendMidiMessage(MidiMessage::midiClock());
@@ -642,14 +653,30 @@ private:
                 {
                     MemoryBlock mem;
                     bool readSuccess = file.loadFileAsData(mem);
-                    if (readSuccess)
+                    if (readSuccess && mem.getSize() > 0)
                     {
-                        sendMidiMessage(MidiMessage(mem.getData(), (int)mem.getSize()));
+                        const uint8* data = (uint8*)mem.getData();
+                        const int data_size = (int)mem.getSize();
+                        const int buffer_size = 256;
+                        sendMidiMessage(MidiMessage(data, data_size));
+
+                        std::cout << "Waiting for typical completion on DIN connections 0% (could be done sooner)" << std::flush;
+                        for (int i = 0; i < data_size; i += buffer_size)
+                        {
+                            int length = std::min(buffer_size, data_size - i);
+                            std::cout << "\rWaiting for typical completion on DIN connections " << (((i + length) * 100) / data_size) << "% (could be done sooner)" << std::flush;
+                            // don't exceed 31250 baudrate (bits per second)
+                            Thread::sleep((buffer_size * 8 * 1000) / 31250);
+                        }
+                        std::cout << "\rWaiting for typical completion on DIN connections 100%" << std::endl;
+
+                        Thread::sleep(((data_size / buffer_size) + 1) * (8 * 1000) / 31250);
                     }
                 }
                 else
                 {
                     std::cerr << "Couldn't find file \"" << path << "\"" << std::endl;
+                    setApplicationReturnValue(EXIT_FAILURE);
                 }
                 break;
             }
@@ -722,13 +749,13 @@ private:
         scenarioStep("Pitch Bend Sensitivity on Master Channel to 7 semitones");
         sendMidiMessage(MidiMessage::controllerEvent(1, 0x64, 0));
         sendMidiMessage(MidiMessage::controllerEvent(1, 0x65, 0));
-        sendMidiMessage(MidiMessage::controllerEvent(1, 0x00, 7));
+        sendMidiMessage(MidiMessage::controllerEvent(1, 0x06, 7));
         
         int note_pbsens = 48;
         scenarioStep(String("Pitch Bend Sensitivity on Member Channels to ") + String(note_pbsens) + " semitones");
         sendMidiMessage(MidiMessage::controllerEvent(2, 0x64, 0));
         sendMidiMessage(MidiMessage::controllerEvent(2, 0x65, 0));
-        sendMidiMessage(MidiMessage::controllerEvent(2, 0x00, note_pbsens));
+        sendMidiMessage(MidiMessage::controllerEvent(2, 0x06, note_pbsens));
         
         scenarioStep("Major C triad C3 E3 G3 on Member Channels with neutral starting expression");
         
@@ -917,8 +944,8 @@ private:
         scenarioStep(String("Pitch Bend Sensitivity on Member Channels to ") + String(note_pbsens) + " semitones");
         sendMidiMessage(MidiMessage::controllerEvent(2, 0x64, 0));
         sendMidiMessage(MidiMessage::controllerEvent(2, 0x65, 0));
-        sendMidiMessage(MidiMessage::controllerEvent(2, 0x00, note_pbsens));
-        
+        sendMidiMessage(MidiMessage::controllerEvent(2, 0x06, note_pbsens));
+
         Thread::sleep(2000);
         
         scenarioStep("Pitch bend back to the original Major C triad C3 E3 G3");
